/*******************************************************************

	PKLib	2.0

	The final multicomponent library

	(c) Copyright 19911995 Marco Savegnago IW3FQG 	- All rights reserved

	This software can be freely copied for non-commercial use.

	Thread and ThreadManager	declaration

	Autore				Data        Note
	------				----		----

	Marco Savegnago     21/11/93    Scritto e collaudato il modulo

*******************************************************************/


#ifndef	__TTHREAD_H
#define	__TTHREAD_H

// 	PKLib 	include

#include <tbobject.h>
#include <tdef.h>

//	CTASK	include

#include <task/tsk.h>

//	CRTL 	include

#include <string.h>

#define __THREAD_SIBILIGNAME	TRUE

#define 	CHECK4CTASK		if(!valid)	return

struct	CTaskElement	:	virtual public TBobject
{
	CTaskElement();
};

inline CTaskElement::CTaskElement()
{
	if(!ctask_active)
			valid=False;
}

class TThread : public CTaskElement
{
public:

	enum Status { Created, Running, Suspended, Finished };

	typedef void ( TThread::*ThreadFnP)();

protected:

	FnP threadStartup;

protected:

	TThread(char *name, word priority = PRI_STD, unsigned stacklen = 2048);

	TThread(const TThread& );

	Boolean Create();

public:

	virtual ~TThread();                // destructor

	int  	Start ();                  // start thread running

	int  	Stop ();                   // start thread running

	void 	Terminate ();              // terminate thread

	int 	wake ();				   // wake thread

	word 	getPriority();			   // get thread priority

	void 	setPriority(word priority);// set thread priority

	void	setFlags(byte flags);

	static void schedule ();           // call the scheduler

	char 	*getName() const;          // get name of thread

// protected:

	static void Execute (void *thread);// Execute a thread

	virtual void  far Run () = 0;  		// main body of thread (pure)

	void 		Wait ();                // wait for thread to terminate

protected:
	tcb*		theTCB;

	char* 		threadName;

	word 		priority;

	unsigned    stackLen;

	volatile 	Status		stat;

	// Friends

	friend class TTasker;
};

//	Inlines

inline TThread::~TThread()                // destructor
{
#ifdef __THREAD_SIBILIGNAME
	free(threadName);
#endif

	Terminate();
}

inline int TThread::Start ()                          // start thread running
{
	if(!Create())
			return False;

	return ::start_task(theTCB);
}

// Stop a running thread

inline int  TThread::Stop ()
{
	return ::stop_task(theTCB);
}

// terminate thread

inline void TThread::Terminate ()
{
	if(valid)
		::kill_task (theTCB);
}

// wake a thread

inline int TThread::wake ()
{
	return ::wake_task(theTCB);
}

inline word TThread::getPriority()
{
	return ::get_priority(theTCB);
}

inline void TThread::setPriority(word priority)
{
	::set_priority(theTCB, priority);
}

inline void	TThread::setFlags(byte flags)
{
	::set_task_flags(theTCB, flags);
}

inline void TThread::schedule ()
{
	::schedule();
}

inline char* TThread::getName () const
{
    return	threadName;
}

#ifdef Use_Template_Collection
#include <collect.h>
typedef TGenList < TThread > TThreadList;
#else
#include <tbobject.h>
typedef TBNSCollection TThreadList;
#endif

class TThreadsManager
{

public:
	TThreadsManager();
	virtual ~TThreadsManager();


	TThreadList* theThreadsList;

	inline		word	NumOfThread();

	virtual 	Boolean startThreads();
	virtual		Boolean stopThreads();
};

inline	word	TThreadsManager::NumOfThread()
{
	return 	theThreadsList->getCount();
}

#endif	// __TTHREAD_H
